#include "2001LandMod.h"

// ==============================================================
// Airfoil coefficient functions
// Return lift, moment and zero-lift drag coefficients as a
// function of angle of attack (alpha or beta)
// ==============================================================

// 1. vertical lift component (wings and body)


void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0, -0.0, -0.0, -0.0,  -0.0,  -0.0, -0.00, 0.0,    0.0,   0.0, 0.0, -0.1, 0.19,  0.35, 0.19, -0.0, -0.0, -0.0, 0.0};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.00 ,0.00 , 0.00 ,0.00 ,      0,-0.00 ,-0.000 ,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.12 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.0, 0.0, 0.0, 0, -0.0, -0.0, -0.0, 0, 0.0, 0.0, 0.0, 0, -0.0, -0.0, -0.0, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.12 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


LANDMOD::LANDMOD (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hCone = oapiLoadMeshGlobal( "2001/LandMod");

	tex   = oapiRegisterExhaustTexture ("exhaust_atrcs");

	banimsset = false;
	RCSMass = 6000.0;
	stage = 0;
	dragset = 0.0;
	drymass = 2000.0;
	EXPAND_SPEED = 0.5;
	DEFLATE_SPEED = 0.05;
	expand_proc = 0.0;
	expand_status = DOOR_CLOSED;
	box_proc = 0.0;
	box_status = DOOR_CLOSED;
	drogue_proc = 0.0;
	drogue_status = DOOR_CLOSED;
	chute_proc = 0.0;
	chute_status = DOOR_CLOSED;
	deflate_proc = 0.0;
	deflate_status = DOOR_CLOSED;


	//                alignment      position      func    chord S   AR   
	CreateAirfoil (LIFT_VERTICAL, _V(0,1,0), VLiftCoeff, 10, 100, 1);
	// wing and body lift+drag components

	CreateAirfoil (LIFT_HORIZONTAL, _V(0,1,0), HLiftCoeff, 10, 100, 1);
	// vertical stabiliser and body lift and drag components


	defaultsuit = false;
//	SetDefSuit();
	
	//		     				child       pos        dir       rot      handle
	Seatpos = CreateAttachment (false, OFS_SEAT,   _V(0,0,-1), _V(0,-1,0), "GX");

/*	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if(( strstr( classname, "EJECTSEAT" ) ) || ( strstr( classname, "NOSUITEJECTSEAT" ) )) {
		RCSMass = 0.0;
		RetroMass = 0.0;
		bExpands = false;
	}

*/

	ph_main = CreatePropellantResource( RCSMass );


}

void LANDMOD::LoadState (FILEHANDLE scn, void *vs)
{
	char *line;
//	int i=0;
      
	ClearMeshesEx();
	while (oapiReadScenario_nextline (scn, line)) 
	{
  		if (!strnicmp (line, "PAYLOAD", 7))
		{
			if (!defaultsuit)
				ParsePayloadString( line );
		}

		else if ( !strnicmp(line, "CONFIGURATION",13))
		{
			sscanf (line+13, "%d", &stage);
		}
		else if (!strnicmp (line, "EXPAND", 6))
		{
			sscanf (line+6, "%lf", &expand_proc);
		}



		else 
		{
			ParseScenarioLineEx (line, vs);
		}

	}

	SetNewStage();

}

void LANDMOD::SaveState (FILEHANDLE scn)
{
	char cbuf[256];

	SaveDefaultState (scn);

	oapiWriteScenario_int (scn, "CONFIGURATION", stage);
	sprintf (cbuf, "%0.4f", expand_proc);
	oapiWriteScenario_string (scn, "EXPAND", cbuf);


	/* save payload lines */
	if (!defaultsuit)
		SaveDefaultStateEx(scn);
}

void LANDMOD::SetStage ()
{

	VECTOR3 pos = _V( 0,0,0 );
	COMMON_MESH tmp;
//	ClearVariableDragElements();



	ClearMeshesEx();
	tmp.MeshHandle(hCone);
	tmp.MeshPosition( pos);
	AddMeshEx( tmp);

	
}
void LANDMOD::SetAnims ()
{
// Stages:
// 0 Box
// 1 Aeroshell expands
// 2 Box rotates and drogue out
// 3 Drogue in, chute out
// 4 Chutes drop

	Draw();
	if (!banimsset) {
		banimsset = true;
	
		anim_expand = CreateAnimation (0.0);
		anim_box = CreateAnimation (0.0);
		anim_drogue = CreateAnimation (0.0);
		anim_chute = CreateAnimation (0.0);
		anim_deflate = CreateAnimation (0.0);

//  Known Groups:

//  0 Nozzles
//  1 Gold Tanks
//  2 Control Box
//  3 Skeleton
//  4 Chute Cans
//  5 Main Chute Doors
//  6 Drogue Chute Doors
//  7 Drogue
//  8 Main Chutes
//  9 Shroud/Tether
//  10 Aeroshield
//  11 Silver Tanks



		static UINT ExpandGrp[1] = {10};
		static MGROUP_SCALE Expand (0, ExpandGrp, 1,
			_V(0,-1.466, 5.237), _V(100.0,100.0,100.0));
		AddAnimationComponent (anim_expand, 0.05, 0.95, &Expand);


		static UINT BoxGrp[11] = {0,1,2,3,4,5,6,7,8,9,11};
		static MGROUP_ROTATE Box (0, BoxGrp, 11,
			_V(0.0,-0.404, 2.0), _V(-1,0,0), (float)(90*RAD));
		static UINT ShroudGrp[1] = {9};
		static MGROUP_SCALE Shroud (0, ShroudGrp, 1,
			_V(0, 4.0, 2.073), _V(20.0,3.0,20.0));
		AddAnimationComponent (anim_box, 0.05, 0.5, &Box);
		AddAnimationComponent (anim_box, 0.5, 0.95, &Shroud);

		static UINT DrogueGrp[1] = {7};
		static MGROUP_SCALE Drogue (0, DrogueGrp, 1,
			_V(0, 3.5295, 2.0834), _V(1000.0,1000.0,1000.0));
		AddAnimationComponent (anim_drogue, 0.05, 0.95, &Drogue);
		static UINT DrogueCvrGrp[1] = {6};
		static MGROUP_SCALE DrogueCvr (0, DrogueCvrGrp, 1,
			_V(0, 3.5295, 2.0834), _V(0.0001,0.0001,0.0001));
		AddAnimationComponent (anim_drogue, 0.05, 0.15, &DrogueCvr);

		static UINT ChuteGrp[1] = {8};
		static MGROUP_SCALE Chute (0, ChuteGrp, 1,
			_V(0, 3.2538, 2.0848), _V(1000.0,1000.0,1000.0));
		AddAnimationComponent (anim_chute, 0.05, 0.95, &Chute);
		static UINT ChuteCvrGrp[1] = {5};
		static MGROUP_SCALE ChuteCvr (0, ChuteCvrGrp, 1,
			_V(0, 3.2538, 2.0848), _V(0.0001,0.0001,0.0001));
		AddAnimationComponent (anim_chute, 0.05, 0.15, &ChuteCvr);

		static UINT DropGrp[11] = {0,1,2,3,4,5,6,7,8,9,11};
		static MGROUP_TRANSLATE Drop (0, DropGrp, 11,
			_V(-8.40,-4.20, 0.0));
		static UINT TiltGrp[1] = {8};
		static MGROUP_ROTATE Tilt (0, TiltGrp, 1,
			_V(-8.40,-0.404, 2.0), _V(0,0,1), (float)(45*RAD));
		static UINT FallGrp[2] = {8,9};
		static MGROUP_SCALE Fall (0, FallGrp, 2,
			_V(0,-0.404, 2.0), _V(1.0,0.0001,1.0));
		static UINT DeflateGrp[1] = {10};
		static MGROUP_SCALE Deflate (0, DeflateGrp, 1,
			_V(0,-1.6, 5.0), _V(1.0,0.0001,1.0));
		AddAnimationComponent (anim_deflate, 0.05, 0.3, &Drop);
		AddAnimationComponent (anim_deflate, 0.3, 0.7, &Tilt);
		AddAnimationComponent (anim_deflate, 0.7, 0.95, &Fall);
		AddAnimationComponent (anim_deflate, 0.8, 0.95, &Deflate);

	CreateVariableDragElement (&expand_proc, 1.25, _V(0, 3.0,  1.0));
	CreateVariableDragElement (&drogue_proc, 8.0, _V(0, 20.0, 0.0));
	CreateVariableDragElement (&chute_proc, 80.0, _V(0, 20.0, 0.0));

	}

	SetAnimation (anim_expand, expand_proc);
	SetAnimation (anim_box, box_proc);
	SetAnimation (anim_drogue, drogue_proc);
	SetAnimation (anim_chute, chute_proc);
	SetAnimation (anim_deflate, deflate_proc);

	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);
	VECTOR3 mesh_dir=_V(0,0.0,6.5);
	SetCameraOffset(mesh_dir);

	
	double MainThrust = 0.0;
	double HoverThrust = 0.0;
	double EjectThrust = 50000;
	double AttThrust = 1000;

	SetEmptyMass( drymass+CalculateMass());
//	if (defaultsuit) SetEmptyMass (350.0);


	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.5, 5, 250, 0.3, 0.5, 16, 2.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 0, 0
	};

	PARTICLESTREAMSPEC exhaust_retro = {
		0, 1.2, 40, 250, 0.1, 0.2, 42, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 0, 0
	};
	exhaust_retro.tex = oapiRegisterParticleTexture ("Contrail2");
	SetRotDrag (_V(1.7,1.7,1.3)); // angular drag
//	SetEnableFocus(FALSE);

/*
	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);

	CreateControlSurface (AIRCTRL_ELEVATOR, 5.0, 1.5, _V( 0, 0,  -15), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_RUDDER,   2.0, 1.5, _V( 0, 3,  -16), AIRCTRL_AXIS_YPOS);
	CreateControlSurface (AIRCTRL_AILERON,  3.0, 1.5, _V( 7,-0.5,-15), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_AILERON,  3.0, 1.5, _V(-7,-0.5,-15), AIRCTRL_AXIS_XNEG);
*/
/*	PARTICLESTREAMSPEC rps = {
		0, 4, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
		PARTICLESTREAMSPEC::ATM_PLIN, 3e9, 12e9
	};
	AddReentryStream (&rps);
*/

//	ClearVariableDragElements();
//	CreateVariableDragElement (&dragset, 20.0, _V(0, 20, 0));

	SetSize (14);
	SetTouchdownPoints(_V(0.0,-1.6,1.5),_V(-1,-1.6,-1.5),_V(1,-1.6,-1.5));
	SetPMI (_V(2.28, 2.15, 4.09));
	if (stage < 2)
		SetCrossSections (_V(20, 20, 10));
	else if (stage == 2)
		SetCrossSections (_V(120, 120, 50));
	else if (stage == 3)
		SetCrossSections (_V(520, 520, 310));
	else
		SetCrossSections (_V(20, 20, 10));
	SetCW (0.7, 0.7, 0.9, 0.9);
	SetRotDrag (_V(0.5, 0.5, 0.3));
	SetPitchMomentScale (-1e-4);
	SetBankMomentScale (-1e-4);
	SetDefaultPropellantResource( ph_main );
//	Burning = false;
	ClearThrusterDefinitions();


	if (stage == 0) {
		eject_engine[0] = CreateThruster (_V(-1.2,0,6.0),    _V( 0.15,0.0,0.9), 
			EjectThrust, ph_main, 4200, 4200);
		AddExhaust(eject_engine[0],3.0,0.3, _V(-1.2,-0.15,6.0), _V(-0.15,0.0,-0.9),tex);
		AddExhaust(eject_engine[0],3.0,0.3, _V(-1.2,-0.67,6.0), _V(-0.15,0.0,-0.9),tex);
		eject_engine[1] = CreateThruster (_V( 1.2,0,6.0),    _V(-0.15,0.0,0.9), 
			EjectThrust, ph_main, 4200, 4200);
		AddExhaust(eject_engine[1],3.0,0.3, _V( 1.2,-0.15,6.0), _V( 0.15,0.0,-0.9),tex);
		AddExhaust(eject_engine[1],3.0,0.3, _V( 1.2,-0.67,6.0), _V( 0.15,0.0,-0.9),tex);
		eject_engine[2] = CreateThruster (_V(0, 1.2,6.0),    _V(0.0,-0.15,0.9), 
			EjectThrust, ph_main, 4200, 4200);
		AddExhaust(eject_engine[2],3.0,0.3, _V( 0.26,0.773,6.0), _V( 0.0,0.15,-0.9),tex);
		AddExhaust(eject_engine[2],3.0,0.3, _V(-0.26,0.773,6.0), _V( 0.0,0.15,-0.9),tex);
		eject_engine[3] = CreateThruster (_V(0,-1.2,6.0),    _V(0.0, 0.15, 0.9), 
			EjectThrust, ph_main, 4200, 4200);
		AddExhaust(eject_engine[3],3.0,0.3, _V( 0.26,-1.6,6.0), _V( 0.0,-0.15,-0.9),tex);
		AddExhaust(eject_engine[3],3.0,0.3, _V(-0.26,-1.6,6.0), _V( 0.0,-0.15,-0.9),tex);
		thg_eject = CreateThrusterGroup(eject_engine,4,THGROUP_MAIN);
	} else if (stage == 1) {
		eject_engine[0] = CreateThruster (_V(0, 1.2,6.0),    _V(0.0,-0.15,0.9), 
			EjectThrust, ph_main, 4200, 4200);
		AddExhaust(eject_engine[0],3.0,0.3, _V( 0.26,0.773,6.0), _V( 0.0,0.15,-0.9),tex);
		AddExhaust(eject_engine[0],3.0,0.3, _V(-0.26,0.773,6.0), _V( 0.0,0.15,-0.9),tex);
		eject_engine[1] = CreateThruster (_V(0,-1.2,6.0),    _V(0.0, 0.15, 0.9), 
			EjectThrust, ph_main, 4200, 4200);
		AddExhaust(eject_engine[1],3.0,0.3, _V( 0.26,-1.6,6.0), _V( 0.0,-0.15,-0.9),tex);
		AddExhaust(eject_engine[1],3.0,0.3, _V(-0.26,-1.6,6.0), _V( 0.0,-0.15,-0.9),tex);
		thg_eject = CreateThrusterGroup(eject_engine,2,THGROUP_MAIN);
	} else if (stage < 4) {
		eject_engine[0] = CreateThruster (_V(-1.2,3.6,0.0),    _V( 0.15,0.9,0.0), 
			EjectThrust, ph_main, 4200, 4200);
		AddExhaust(eject_engine[0],3.0,0.3, _V(-1.2,3.6,1.70), _V(-0.15,-0.9,0),tex);
		AddExhaust(eject_engine[0],3.0,0.3, _V(-1.2,3.6,2.20), _V(-0.15,-0.9,0),tex);
		eject_engine[1] = CreateThruster (_V( 1.2,3.6,0.0),    _V(-0.15,0.9,0.0), 
			EjectThrust, ph_main, 4200, 4200);
		AddExhaust(eject_engine[1],3.0,0.3, _V( 1.2,3.6,1.70), _V( 0.15,-0.9,0.0),tex);
		AddExhaust(eject_engine[1],3.0,0.3, _V( 1.2,3.6,2.20), _V( 0.15,-0.9,0.0),tex);
		eject_engine[2] = CreateThruster (_V(0, 3.6,-1.2),    _V(0.0,0.9, 0.15), 
			EjectThrust, ph_main, 4200, 4200);
		AddExhaust(eject_engine[2],3.0,0.3, _V( 0.26,3.6,0.82), _V( 0.0,-0.9,-0.15),tex);
		AddExhaust(eject_engine[2],3.0,0.3, _V(-0.26,3.6,0.82), _V( 0.0,-0.9,-0.15),tex);
		eject_engine[3] = CreateThruster (_V(0,3.6,1.2),    _V(0.0, 0.9,-0.15), 
			EjectThrust, ph_main, 4200, 4200);
		AddExhaust(eject_engine[3],3.0,0.3, _V( 0.26,3.6,3.17), _V( 0.0,-0.9, 0.15),tex);	
		AddExhaust(eject_engine[3],3.0,0.3, _V(-0.26,3.6,3.17), _V( 0.0,-0.9, 0.15),tex);	
		thg_eject = CreateThrusterGroup(eject_engine,4,THGROUP_MAIN);
	}


//	AddExhaustStream (eject_engine[0], &contrail);

//	retex = oapiRegisterReentryTexture("reentry");
//	SetReentryTexture(retex,6e5,1,1);



// 5 Dock #3



/*
	th_main[0] = CreateThruster (_V(0,0.0,-15.5),    _V( 0.0,0.0,1.0), 
		MainThrust, ph_main, 4e4, 4e5);

	th_main[1] = CreateThruster (_V(-3.7,0.0,-11.5), _V( 0.2,0.0,0.87), 
		MainThrust/2, ph_main, 4e4, 4e5);

	th_main[2] = CreateThruster (_V( 3.7,0.0,-11.5), _V( -0.2,0.0,0.87), 
		MainThrust/2, ph_main, 4e4, 4e5);

	thg_main = CreateThrusterGroup( th_main, 3, THGROUP_MAIN );
	AddExhaustStream (th_main[0], _V( 0.0,  0, -25.0), &exhaust_main);
	AddExhaustStream (th_main[1], _V(-2.6, 0.0,-20.0), &exhaust_main);
	AddExhaustStream (th_main[2], _V( 2.6, 0.0,-20.0), &exhaust_main);

	AddExhaust (th_main[0], 10.0, 1.0, tex);
	AddExhaust (th_main[1], 10.0, 0.5, tex);
	AddExhaust (th_main[2], 10.0, 0.5, tex);

	th_hover[0] = CreateThruster (_V(0,0,0), _V(0,1,0), HoverThrust, ph_main, 4e4, 4e5);
	thg_hover = CreateThrusterGroup (th_hover, 1, THGROUP_HOVER);

	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 1.2),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 1.2),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 4.55),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 4.55),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.0, 7.25),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.0, 7.25),_V(  0.0, -1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V( -0.5,   -1.2, -6.8),_V(  0.0,-1.0, 0.0), tex);
	AddExhaust (th_hover[0], 4.0, 0.2 ,_V(  0.5,   -1.2, -6.8),_V(  0.0,-1.0, 0.0), tex);
*/
	// set of attitude thrusters (idealised). The arrangement is such that no angular
	// momentum is created in linear mode, and no linear momentum is created in rotational mode.

	THRUSTER_HANDLE th_att_rot[4], th_att_lin[4];
	th_att_rot[0] = CreateThruster (_V(0,0, 15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[1] = CreateThruster (_V(0,0,-15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[2] = CreateThruster (_V(0,0, 15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[3] = CreateThruster (_V(0,0,-15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_PITCHUP);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_PITCHDOWN);

	th_att_lin[0] = CreateThruster (_V(0,0, 15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[3] = CreateThruster (_V(0,0,-15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[2] = CreateThruster (_V(0,0, 15.5), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[1] = CreateThruster (_V(0,0,-15.5), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_UP);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_DOWN);

	th_att_rot[0] = CreateThruster (_V(0,0, 15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[1] = CreateThruster (_V(0,0,-15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[2] = CreateThruster (_V(0,0, 15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[3] = CreateThruster (_V(0,0,-15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_YAWLEFT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_YAWRIGHT);

	th_att_lin[0] = CreateThruster (_V(0,0, 15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[3] = CreateThruster (_V(0,0,-15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[2] = CreateThruster (_V(0,0, 15.5), _V( 1,0,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[1] = CreateThruster (_V(0,0,-15.5), _V(-1,0,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_LEFT);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_RIGHT);

	th_att_rot[0] = CreateThruster (_V( 2.7,0,0), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[1] = CreateThruster (_V(-2.7,0,0), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[2] = CreateThruster (_V(-2.7,0,0), _V(0, 1,0), AttThrust, ph_main, 4e4, 4e5);
	th_att_rot[3] = CreateThruster (_V( 2.7,0,0), _V(0,-1,0), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_BANKLEFT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_BANKRIGHT);

	th_att_lin[0] = CreateThruster (_V(0,0,-16), _V(0,0, 1), AttThrust, ph_main, 4e4, 4e5);
	th_att_lin[1] = CreateThruster (_V(0,0, 16), _V(0,0,-1), AttThrust, ph_main, 4e4, 4e5);
	CreateThrusterGroup (th_att_lin,   1, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_att_lin+1, 1, THGROUP_ATT_BACK);


}

void LANDMOD::SetClassCaps (FILEHANDLE cfg)
{

	CheckCentralMessage();

	SetStage();
	SetAnims();
}


void LANDMOD::TimeStep( double simt )
{

	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ; {
			SetStage();
			Draw();
		}
	}

	OBJHANDLE hV = GetAttachmentStatus(Seatpos);
	if (hV) {
		VESSEL *v = oapiGetVesselInterface (hV);
		double strappedmass = v->GetMass();
		SetEmptyMass (drymass+strappedmass+CalculateMass());
	}


	if (stage == 3)
		if ((GetAltitude() < 5.0) && (GetMachNumber() < 0.05)) {
			stage = 4;
			SetNewStage();
			SetThrusterGroupLevel(THGROUP_MAIN, 0.0);
		}
//		else if (GetAltitude() < 200.0)
//			SetThrusterGroupLevel(THGROUP_MAIN, 1.0);

	if (stage == 2)
		if ((GetAltitude() < 15000.0) && (GetMachNumber() < 5.0)) {
			stage = 3;
			SetNewStage();
		}
		
	if (stage == 1)
		if ((GetAltitude() < 35000.0) && (GetMachNumber() < 11.0)) {
			stage = 2;
			SetNewStage();
		}

	if (stage == 0)
		if (GetAltitude() < 120000.0) {
			VECTOR3 horiz;
			GetHorizonAirspeedVector(horiz);
			if (horiz.y < -500.0) {
				stage = 1;
				SetNewStage();
				StrapUnstrap();
				StrapUnstrap();				// Re-strap in so that the re-entry trail of the payload is close enough
			}
		}



	if (stage == 1) {
		SetAnimation (anim_expand, expand_proc);
		double da = oapiGetSimStep() * EXPAND_SPEED;
		if ((expand_proc > 0.0) && (expand_proc < 1.0)) {
			expand_proc = expand_proc + da;
		}
		if (expand_proc >= 1.0) {
			Expanding = false;
			expand_proc = 1.0;
		}
	}

	if (stage == 4) {
		SetAnimation (anim_deflate, deflate_proc);
		double da = oapiGetSimStep() * DEFLATE_SPEED;
		if ((deflate_proc > 0.0) && (deflate_proc < 1.0)) {
			deflate_proc = deflate_proc + da;
		}
		if (deflate_proc >= 1.0) {
			deflate_proc = 1.0;
			StrapUnstrap();
		}
	}

}

void LANDMOD::SetNewStage()
{

	if (stage == 1)
		expand_proc = oapiGetSimStep() * EXPAND_SPEED;
	else if (stage == 2) {
		box_proc = 1.0;
		drogue_proc = 1.0;
	} else if (stage == 3) {
		box_proc = 1.0;
		drogue_proc = 0.0;
		chute_proc = 1.0;
	} else if (stage == 4) {
		box_proc = 1.0;
		drogue_proc = 0.0;
		chute_proc = 1.0;
		deflate_proc = oapiGetSimStep() * DEFLATE_SPEED;
	}


	SetAnims();
	SetStage();
	Draw();
}

void LANDMOD::StrapUnstrap()
{

			VECTOR3 gpos, grms, pos, dir, rot;
			Local2Global (OFS_SEAT, grms);  // global position of RMS tip
			
			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...

			for (DWORD i = 0; i < oapiGetVesselCount(); i++) {
				OBJHANDLE hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < oapiGetSize (hV)) { // in range
					
//					sprintf(oapiDebugString(),"Distance is too far to strap in!");

					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (true);
					for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
						ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
						const char *id = v->GetAttachmentId (hAtt);
//						if (strncmp (id, "GX", 2)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						if (dist (gpos, grms) < MAX_GRAPPLING_DIST) { // found one!
							// check whether satellite is currently clamped into payload bay
							if (hV == GetAttachmentStatus (Seatpos))
							{
								DetachChild (Seatpos);
//								sprintf(oapiDebugString(),"Unstrapped");
//								oapiSetFocusObject (hV);
								SetEmptyMass (drymass);
							}
							else
							{
								AttachChild (hV, Seatpos, hAtt);
//								sprintf(oapiDebugString(),"Strapped in!");	
							}
							return;
						}
					}
				}
			
			}
}

int LANDMOD::ConsumeKey ( const char *keystate)
{
	return 0;
}

int LANDMOD::ConsumeBufferedKey ( DWORD dwKey, bool fPressed, const char *keystate)
{

		if (KEYDOWN (keystate, OAPI_KEY_K)) 
		{ // "Unfold"
			if (oapiAcceptDelayedKey (OAPI_KEY_K, 1.0))
			{
			//	OperateDoors (door_status == DOOR_CLOSED || door_status == DOOR_CLOSING ?
			//		DOOR_OPENING : DOOR_CLOSING);
//				Expanding = true;
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
//				OverridePayloadVelocity(_V(0,0,0));
//				depV = DeployPayload();
				SetStage();
				Draw();
//				SetEmptyMass( 100.0+CalculateMass());
//				if (depV) {
//					hVessel=oapiGetVesselByName(depV->GetName());
//					oapiSetFocusObject(hVessel);
//				}

			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_S)) { // "Strap/Unstrap" 
			if (oapiAcceptDelayedKey (OAPI_KEY_S, 1.0)) {
				StrapUnstrap();
				SetStage();
				Draw();
			}
		}

/*
		if (KEYDOWN (keystate, OAPI_KEY_E)) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
			{
			

				VESSELSTATUS vs;
				OBJHANDLE hvessel;
				GetStatus (vs);
				vs.eng_main = vs.eng_hovr = 0.0;
				vs.fuel=1.0;
				vs.status=0;
				VECTOR3 ofs = _V(3.5,0.0,17.25);
				Local2Rel (ofs, vs.rpos);
				char name[256];
				numevas++;
				sprintf(name, "EVA-%d", numevas);
	
				VECTOR3 vel = _V(0,0.2,1.0);
				VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				VESSEL::Create (name, "EVA", vs);
				hvessel=oapiGetVesselByName(name);
				oapiSetFocusObject(hvessel);
	
			}
			return 1;
		}
*/

		return 0;
}




/* basic vessel class setup */

void LANDMOD::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void LANDMOD::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void LANDMOD::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void LANDMOD::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void LANDMOD::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetAnims();
	SetStage();
	Draw();
}

void LANDMOD::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetAnims();
	SetStage();
	Draw();
}

int LANDMOD::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new LANDMOD (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (LANDMOD*)vessel;
}

